==> makumba-0_8_1_1-src.zip: makumba/classes/org/makumba/commons/SoftNamedResources.java <==
///////////////////////////////
//  Makumba, Makumba tag library
//  Copyright (C) 2000-2003  http://www.makumba.org
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//  -------------
//  $Id: SoftNamedResources.java 2545 2008-06-11 18:45:43Z rosso_nero $
//  $Name$
/////////////////////////////////////

package org.makumba.commons;

import java.lang.ref.SoftReference;

/**
 * A NamedResources that keeps its resources as soft references
 * 
 * @see org.makumba.commons.NamedResources
 */
public class SoftNamedResources extends NamedResources {
    private static final long serialVersionUID = 1L;

    public SoftNamedResources(String name, NamedResourceFactory f) {
        super(name, f);
    }

    @Override
    protected NameValue getNameValue(Object name, Object hash) {
        NameValue nv = null;
        cleanCleared();
        SoftReference sr = (SoftReference) values.get(hash);
        if (sr == null || (nv = (NameValue) sr.get()) == null) {
            values.put(hash, new SoftReference(nv = new NameValue(name, hash, f)));
            misses++;
        } else {
            hits++;
        }
        return nv;
    }

    @Override
    public String getName() {
        return name + " (soft cache)";
    }

    /**
     * remove the residue data structures which were used to refer the resources that were cleared by the garbage
     * collector
     */
    void cleanCleared() {
        for (java.util.Iterator i = values.keySet().iterator(); i.hasNext();) {
            SoftReference sr = (SoftReference) values.get(i.next());
            if (sr.get() == null) {
                i.remove();
            }
        }
    }

    @Override
    public synchronized int size() {
        cleanCleared();
        return super.size();
    }
}

==> makumba-0_8_1_9_1-src.zip: makumba/classes/org/makumba/commons/SoftNamedResources.java <==
///////////////////////////////
//  Makumba, Makumba tag library
//  Copyright (C) 2000-2003  http://www.makumba.org
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//  -------------
//  $Id: SoftNamedResources.java 3925 2009-06-09 12:22:22Z manuel_gay $
//  $Name$
/////////////////////////////////////

package org.makumba.commons;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.SoftReference;

/**
 * A NamedResources that keeps its resources as soft references
 * 
 * @see org.makumba.commons.NamedResources
 */
public class SoftNamedResources extends NamedResources {
    private static final long serialVersionUID = 1L;
    private ReferenceQueue<NameValue> queue= new ReferenceQueue<NameValue>();
    private int diff=0;
    
    public SoftNamedResources(String name, NamedResourceFactory f) {
        super(name, f);
    }

    @Override
    protected NameValue getNameValue(Object name, Object hash) {
        NameValue nv = null;
        SoftReference<NameValue> sr = (SoftReference<NameValue>) values.get(hash);
        if (sr == null || (nv = sr.get()) == null){ 
            if(sr!=null && nv==null)
                diff--;
            values.put(hash, new SoftReference<NameValue>(nv = new NameValue(name, hash, f), queue));
            misses++;
        } else {
            hits++;
        }
        return nv;
    }

    @Override
    public String getName() {
        return name + " (soft cache)";
    }

    @Override
    public synchronized int size() {
        while(queue.poll()!=null){
            try {
                queue.remove();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            diff++;
        }
        
        // sanity check
//        int diff2=0;
//        for (Object o:values.values())
//            if (((SoftReference<NameValue>) o).get() == null) 
//                diff2++;
//        if(diff2!=diff)
//            throw new IllegalStateException(diff+ "<>"+diff2+" out of " +super.size());
        // end check
    
        return super.size()-diff;
    }
}
